package com.bureau.devicefingerprint

import android.Manifest
import android.app.Application
import android.content.Context
import android.content.pm.PackageManager
import android.util.Log
import androidx.annotation.Keep
import androidx.core.app.ActivityCompat
import com.bureau.base.AppInfo
import com.bureau.base.DevUtils
import com.bureau.base.Environment
import com.bureau.base.LocalErrorCodes
import com.bureau.behavioralbiometrics.UserBehaviorCapture
import com.bureau.devicefingerprint.data.remote.RemoteDataSource
import com.bureau.devicefingerprint.data.remote.RemoteDataSourceImpl
import com.bureau.devicefingerprint.models.Callback
import com.bureau.devicefingerprint.models.CredentialBuilder
import com.bureau.devicefingerprint.models.ErrorResponse
import com.bureau.devicefingerprint.models.SubmitResponse
import com.bureau.devicefingerprint.models.devicedataholder.CodecInfo
import com.bureau.devicefingerprint.models.devicedataholder.DeviceDataHolder
import com.bureau.devicefingerprint.models.devicedataholder.MediaDrmId
import com.bureau.devicefingerprint.models.devicedataholder.RealTimeDatHolder
import com.bureau.devicefingerprint.tools.DataCallback
import com.bureau.devicefingerprint.tools.RealTimeDataCallback
import com.bureau.devicefingerprint.tools.RealTimeDataChannel
import com.bureau.devicefingerprint.tools.SharedPreferenceHelper
import com.bureau.devicefingerprint.usecases.GetEncryptedDeviceDataUseCase
import io.sentry.Sentry
import io.sentry.SentryOptions
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

@Keep
class BureauAPI private constructor(private val application: Application, private val clientId: String, private val sessionId: String, private val environment: Environment, private val enableBehavioralBiometrics: Boolean, private val isDev: Boolean = false) {
    private val remoteDataSource: RemoteDataSource by lazy { RemoteDataSourceImpl(environment, isDev) }
    private var isStartSessionCalled = false
    private var credentialBuilder: CredentialBuilder? = CredentialBuilder()
    private var deviceData: DeviceDataHolder? = null
    private var dataMapper: DataMapper = DataMapper(application)
    private var dataMonitoring: DataMonitoring = DataMonitoring(application)
    private var permissionList: ArrayList<String> = arrayListOf(Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION)
    private var isRealTimeDataEnabled: Boolean = false
    private var realTimeDataHolder: RealTimeDatHolder? = null
    private var realTimeDataCallback: RealTimeDataCallback? = null
    private var userBehaviorCapture: UserBehaviorCapture? = null
    private var getEncryptedDeviceDataUseCase = GetEncryptedDeviceDataUseCase(application)

    init {
        if (!Sentry.isEnabled()) {
            sendExceptionToSentry = true
            Sentry.init { options: SentryOptions ->
                options.dsn = BuildConfig.SENTRY_DSN_DEVICE
                // Set tracesSampleRate to 1.0 to capture 100% of transactions for performance monitoring.
                // We recommend adjusting this value in production.
                options.tracesSampleRate = 1.0
                // When first trying Sentry it's good to see what the SDK is doing:
                // options.isDebug = true
                options.environment = if (BuildConfig.DEBUG) "SANDBOX" else "PRODUCTION"
                options.setTag("isDebug", BuildConfig.DEBUG.toString())
                options.setTag("buildType", BuildConfig.BUILD_TYPE)
                options.setTag("libraryPackageName", BuildConfig.LIBRARY_PACKAGE_NAME)
                options.setTag("libraryVersion", BuildConfig.LIBRARY_VERSION_NAME)
                options.release = BuildConfig.LIBRARY_VERSION_NAME
                options.proguardUuid = "d14834aa-8f6e-11ee-b9d1-0242ac120002"
                options.addBundleId("f2e831c4-8f59-11ee-b9d1-0242ac120002")
            }
            setupGlobalTags(AppInfo(application))
        }
        if (clientId.isEmpty()) {
            throw Error("Credential id should not be empty")
        }

        credentialBuilder?.sessionId = sessionId

        if (enableBehavioralBiometrics) {
            userBehaviorCapture = UserBehaviorCapture.init(application, environment, clientId, sessionId, sendExceptionToSentry, isDev)
        }
    }

    private fun setupGlobalTags(appInfo: AppInfo) {
        Sentry.setTag("client.appPackage", appInfo.getPackageName())
        Sentry.setTag("client.appVersion", appInfo.getVersion())
        Sentry.setTag("client.appName", appInfo.getAppName())
        Sentry.setTag("appBuildType", if (appInfo.isInDebugMode()) "debug" else "release")
    }

    fun startSubSession(subSessionId: String) {
        try {
            setUpMandatoryDataCollection()
            isStartSessionCalled = true
            userBehaviorCapture?.startSubSession(subSessionId, credentialBuilder?.userId ?: "")
        } catch (throwable: Throwable) {
            handleException(throwable)
        }
    }

    fun setUserId(userId: String) {
        try {
            credentialBuilder?.userId = userId
        } catch (throwable: Throwable) {
            handleException(throwable)
        }
    }

    fun setFlow(flow: String) {
        try {
            credentialBuilder?.flow = flow
        } catch (throwable: Throwable) {
            handleException(throwable)
        }
    }

    fun stopSubSession() {
        try {
            userBehaviorCapture?.stopSubSession()
        } catch (throwable: Throwable) {
            handleException(throwable)
        }
    }

    private fun collectData() {
        var hasLocationPermission = false
        if (hasPermissions(application)) {
            hasLocationPermission = true
        }
        deviceData = dataMapper.getSynchronisedOSData(hasLocationPermission)
    }

    fun setRealtimeCallback(callback: RealTimeDataCallback) {
        try {
            this.realTimeDataCallback = callback
            isRealTimeDataEnabled = true
            realTimeDataHolder = RealTimeDatHolder()
            Log.i("DeviceService", "Real Time Data Monitoring Started")
            dataMonitoring.registerDataMonitoring(object : RealTimeDataChannel {
                override fun emitVPNStatus(item: Boolean) {
                    realTimeDataHolder?.vpnStatus_ = item
                    if (item) {
                        SharedPreferenceHelper.setSharedPreferenceBoolean(
                            application,
                            SharedPreferenceHelper.isPastVPNData,
                            value = true,
                            isDataMonitoring = true
                        )
                    }
                    CoroutineScope(Dispatchers.Main).launch {
                        realTimeDataCallback?.onVPNStatusChange(item)
                    }
                }

                override fun emitMockGPSStatus(item: Boolean) {
                    realTimeDataHolder?.mockGPSStatus_ = item
                    if (item) {
                        SharedPreferenceHelper.setSharedPreferenceBoolean(
                            application,
                            SharedPreferenceHelper.isPastMockGPSData,
                            value = true,
                            isDataMonitoring = true
                        )
                    }
                    CoroutineScope(Dispatchers.Main).launch {
                        realTimeDataCallback?.onMockGpsStatusChange(item)
                    }
                }

                override fun emitDebuggerStatus(item: Boolean) {
                    realTimeDataHolder?.debuggerStatus_ = item
                    if (item) {
                        SharedPreferenceHelper.setSharedPreferenceBoolean(
                            application,
                            SharedPreferenceHelper.isPastDebuggerData,
                            value = true,
                            isDataMonitoring = true
                        )
                    }
                    CoroutineScope(Dispatchers.Main).launch {
                        realTimeDataCallback?.onDebuggerStatusChange(item)
                    }
                }
            })
        } catch (throwable: Throwable) {
            handleException(throwable)
        }
    }

    fun submit(callback: DataCallback) {
        try {
            if (sessionId.isEmpty()) {
                callback.onError(
                    ErrorResponse(
                        "Error : SessionId should not be empty",
                        LocalErrorCodes.ERROR_CODE_SESSION_ID_EMPTY.toString()
                    )
                )
            } else {
                stopSubSession()
                submitData(callback, sessionId)
            }
        } catch (throwable: Throwable) {
            handleException(throwable)
            callback.onError(
                ErrorResponse(
                    "Fatal error",
                    LocalErrorCodes.ERROR_CODE_GENERAL_THROWABLE.toString()
                )
            )
        }
    }

    private fun submitData(callback: DataCallback, sessionId: String) {
        if (!isStartSessionCalled) {
            setUpMandatoryDataCollection()
        }

        CoroutineScope(Dispatchers.IO).launch {
            val job1 = CoroutineScope(Dispatchers.IO).launch {
                deviceData =
                    deviceData?.let {
                        dataMapper.getAsyncOSData(
                            credentialBuilder = credentialBuilder,
                            it
                        )
                    }
            }

            var mediaDrmId: MediaDrmId? = null
            val job2 = CoroutineScope(Dispatchers.IO).launch {
                mediaDrmId = dataMapper.getMediaDrmId()
            }

            var codecInfo: CodecInfo? = null
            val job3 = CoroutineScope(Dispatchers.IO).launch {
                codecInfo = dataMapper.getMediaCodecInfo()
            }

            var isHookingDetected = false
            val job4 = CoroutineScope(Dispatchers.IO).launch {
                isHookingDetected = dataMapper.getIsHookingDetected()
                Log.d("balatag", "submitData: $isHookingDetected")
            }

            job1.join()
            job2.join()
            job3.join()
            job4.join()
            deviceData?.mediaDrmId_ = mediaDrmId
            deviceData?.codecInfo_ = codecInfo
            deviceData?.isHookingDetected = isHookingDetected

            deviceData = deviceData?.let { dataMapper.getAsynchronousIds(it) }
            deviceData = deviceData?.let { dataMapper.addGeocodeData(it) }
            if (isRealTimeDataEnabled) {
                deviceData?.realTimeDataInfo_ = realTimeDataHolder
            }

            sendData(sessionId, callback, "")
        }
    }

    private suspend fun sendData(
        sessionId: String,
        callback: DataCallback,
        networkSessionId: String
    ) {
        deviceData?.let { dataMapper.addPastRealData(it) }

        if (sessionId.isNotEmpty()) {
            deviceData?.sessionId_ = sessionId
        }
        if (networkSessionId.isNotEmpty()) {
            deviceData?.networkSessionId_ = networkSessionId
        }

        val encryptedData = getEncryptedDeviceDataUseCase(deviceData)
        if (encryptedData.eaIV.isEmpty() || encryptedData.eaKey.isEmpty()) {
            callback.onError(ErrorResponse("invalid data", LocalErrorCodes.ERROR_CODE_EA_KEYS_EMPTY.toString()))
            return
        }
        remoteDataSource.sendData(
            encryptedDeviceData = getEncryptedDeviceDataUseCase(deviceData),
            object : Callback {
                override fun getResult(isSuccess: Boolean?) {
                    SharedPreferenceHelper.clearAllDataPref(application)

                    CoroutineScope(Dispatchers.Main).launch {
                        isStartSessionCalled = false
                        callback.onResult(SubmitResponse("Success"))
                    }
                }

                override fun getError(error: String?, errorCode: String) {
                    SharedPreferenceHelper.clearAllDataPref(application)
                    CoroutineScope(Dispatchers.Main).launch {
                        isStartSessionCalled = false

                        callback.onError(ErrorResponse(error ?: "Error", errorCode))
                    }
                }
            },
            clientId
        )
    }

    private fun hasPermissions(context: Context): Boolean {
        permissionList.forEach {
            if (ActivityCompat.checkSelfPermission(
                    context,
                    it
                ) != PackageManager.PERMISSION_GRANTED
            ) {
                return false
            }
        }
        return true
    }

    private fun checkErrors() {
        if (sessionId.isEmpty()) {
            throw Error("Session id should not be empty")
        }
        if (credentialBuilder?.userId?.isEmpty() == true) {
            Log.w("Prism SDK", "User Id received is empty")
        }
    }

    private fun setUpMandatoryDataCollection() {
        checkErrors()
        collectData()
    }

    fun removeRealTimeCallBack() {
        try {
            if (isRealTimeDataEnabled) {
                this.realTimeDataCallback = null
                isRealTimeDataEnabled = false
                dataMonitoring.unRegisterDataMonitoring()
                Log.i("DeviceService", "Real Time Data Monitoring Stopped")
            }
        } catch (throwable: Throwable) {
            handleException(throwable)
        }
    }

    @Keep
    companion object {

        private var sendExceptionToSentry = false
        fun getBureauApiInstance(application: Application, clientId: String, sessionId: String, environment: Environment, enableBehavioralBiometrics: Boolean = false): BureauAPI {
            return BureauAPI(application, clientId, sessionId, environment, enableBehavioralBiometrics, DevUtils.isDev)
        }

        internal fun handleException(throwable: Throwable) {
            try {
                if (sendExceptionToSentry) {
                    Sentry.captureException(throwable)
                }
            } catch (throwable: Throwable) {
                Log.e("Sentry", "Failed to report a crash.", throwable)
            }
            /*if (BuildConfig.DEBUG) {
                Log.e("Sentry", "Something went wrong.", throwable)
                throw RuntimeException(throwable)
            }*/
        }
    }
}
