package com.bureau.devicefingerprint

import android.Manifest
import android.content.Context
import android.content.pm.PackageManager
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.net.NetworkRequest
import android.os.Bundle
import android.os.Debug
import android.util.Log
import androidx.core.app.ActivityCompat
import com.bureau.devicefingerprint.datacollectors.NetworkInfoProvider
import com.bureau.devicefingerprint.tools.RealTimeDataChannel
import com.bureau.devicefingerprint.tools.executeSafe
import kotlinx.coroutines.*

class DataMonitoring(var context: Context) {
    private var locationManager: LocationManager = context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
    private var connectivityManager: ConnectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
    private var job: Job? = null
    private var vpnCallback: ConnectivityManager.NetworkCallback? = null
    private var locationCallback: LocationListener? = null

    fun registerDataMonitoring(channel: RealTimeDataChannel) {
        CoroutineScope(Dispatchers.Main).launch {
            registerRealTimeVpnCallback(channel)
            registerRealTimeMockGPSCallback(channel)
            registerRealTimeDebuggerCallback(channel)
        }
    }

    fun unRegisterDataMonitoring() {
        closeListeners()
    }

    private fun registerRealTimeVpnCallback(channel: RealTimeDataChannel) {
        try {
            vpnCallback = object : ConnectivityManager.NetworkCallback() {
                override fun onAvailable(network: Network) {
                    channel.emitVPNStatus(true)
                }

                override fun onLost(network: Network) {
                    channel.emitVPNStatus(false)
                }
            }
            vpnCallback?.let {
                connectivityManager.registerNetworkCallback(
                    // I have to investigate on this builder!
                    NetworkRequest.Builder()
                        .addTransportType(NetworkCapabilities.TRANSPORT_VPN)
                        .removeCapability(NetworkCapabilities.NET_CAPABILITY_NOT_VPN)
                        .build(),
                    it
                )
                if (NetworkInfoProvider(context).isConnectedToVPN()) {
                    channel.emitVPNStatus(true)
                } else {
                    channel.emitVPNStatus(false)
                }
            }
        } catch (e: Exception) {
            BureauAPI.handleException(e)
            Log.w("DeviceService", "Failed to monitor VPN $e")
        } catch (er: Error) {
            BureauAPI.handleException(er)
            Log.w("DeviceService", "Failed to monitor VPN $er")
        }
    }

    private fun registerRealTimeMockGPSCallback(channel: RealTimeDataChannel) {
        try {
            locationCallback = object : LocationListener {
                override fun onProviderDisabled(provider: String) {
                }

                override fun onProviderEnabled(provider: String) {
                }

                override fun onStatusChanged(provider: String, status: Int, extras: Bundle) {}
                override fun onLocationChanged(location: Location) {
                    channel.emitMockGPSStatus(isMockLocationEnabled(location))
                }
            }
            if (isLocationEnabled()) {
                if (ActivityCompat.checkSelfPermission(
                        context,
                        Manifest.permission.ACCESS_FINE_LOCATION
                    ) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(
                            context,
                            Manifest.permission.ACCESS_COARSE_LOCATION
                        ) == PackageManager.PERMISSION_GRANTED
                ) {
                    locationCallback?.let {
                        locationManager.requestLocationUpdates(
                            LocationManager.GPS_PROVIDER,
                            2000,
                            10f,
                            it
                        )
                    }
                }
            }
        } catch (e: Exception) {
            BureauAPI.handleException(e)
            Log.w("DeviceService", "Failed to monitor location $e")
        } catch (er: Error) {
            BureauAPI.handleException(er)
            Log.w("DeviceService", "Failed to monitor location $er")
        }
    }

    private fun registerRealTimeDebuggerCallback(channel: RealTimeDataChannel) {
        try {
            job = CoroutineScope(Dispatchers.IO).launch {
                channel.emitDebuggerStatus(
                    executeSafe({
                        Debug.isDebuggerConnected()
                    }, false)
                )
                Debug.waitForDebugger()
                channel.emitDebuggerStatus(
                    executeSafe({
                        Debug.isDebuggerConnected()
                    }, false)
                )
            }
        } catch (e: Exception) {
            BureauAPI.handleException(e)
            Log.w("DeviceService", "Failed to monitor debugger $e")
        } catch (er: Error) {
            BureauAPI.handleException(er)
            Log.w("DeviceService", "Failed to monitor debugger $er")
        }
    }

    private fun isMockLocationEnabled(location: Location?): Boolean {
        return executeSafe({
            (
                location?.isFromMockProvider
                ) == true
        }, false)
    }

    private fun isLocationEnabled(): Boolean {
        return executeSafe({
            val gpsEnabled = try {
                locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
            } catch (ex: Exception) {
                BureauAPI.handleException(ex)
                false
            }

            val networkEnabled = try {
                locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
            } catch (ex: Exception) {
                BureauAPI.handleException(ex)
                false
            }
            return@executeSafe gpsEnabled && networkEnabled
        }, false)
    }

    private fun closeListeners() {
        locationCallback?.let { locationManager.removeUpdates(it) }
        locationCallback = null
        vpnCallback?.let { connectivityManager.unregisterNetworkCallback(it) }
        vpnCallback = null
        CoroutineScope(Dispatchers.IO).launch {
            job?.cancelAndJoin()
        }
    }
}
