package com.bureau.devicefingerprint.datacollectors

import android.Manifest
import android.content.Context
import android.content.pm.PackageManager
import android.location.*
import android.os.Bundle
import androidx.core.app.ActivityCompat
import com.bureau.devicefingerprint.BureauAPI
import com.bureau.devicefingerprint.models.LocationInfoHolder
import com.bureau.devicefingerprint.tools.executeSafe
import java.io.IOException
import java.util.*

class LocationInfoProvider(var context: Context) {
    private val lm: LocationManager = context.applicationContext.getSystemService(Context.LOCATION_SERVICE) as LocationManager
    private var locationListener: LocationListener? = null
    var currentLocation: Location? = null
    var lastKnownLocation: Location? = null

    fun getAreGpsAndNetworkEnabled(): Pair<Boolean, Boolean> {
        return executeSafe({
            val gpsEnabled: Boolean = try {
                lm.isProviderEnabled(LocationManager.GPS_PROVIDER)
            } catch (ex: Exception) {
                BureauAPI.handleException(ex)
                false
            }

            val networkEnabled: Boolean = try {
                lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
            } catch (ex: Exception) {
                BureauAPI.handleException(ex)
                false
            }
            return@executeSafe Pair(gpsEnabled, networkEnabled)
        }, Pair(false, false))
    }

    fun getIsLocationEnabled(): Boolean {
        val (gpsEnabled, networkEnabled) = getAreGpsAndNetworkEnabled()
        return gpsEnabled && networkEnabled
    }

    private fun isMockLocationEnabled(location: Location?): Boolean {
        return executeSafe({
            (
                location?.isFromMockProvider
                ) == true
        }, false)
    }

    fun getKnownLocation(): LocationInfoHolder {
        if (getIsLocationEnabled()) {
            locationListener = MyLocationListener()
            if (ActivityCompat.checkSelfPermission(
                    context,
                    Manifest.permission.ACCESS_FINE_LOCATION
                ) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(
                        context,
                        Manifest.permission.ACCESS_COARSE_LOCATION
                    ) == PackageManager.PERMISSION_GRANTED
            ) {
                lm.requestLocationUpdates(
                    LocationManager.GPS_PROVIDER,
                    1000,
                    1f,
                    locationListener as MyLocationListener
                )
                currentLocation = (locationListener as MyLocationListener).location
                return LocationInfoHolder(
                    longitude =
                    executeSafe({ (locationListener as MyLocationListener).longitudeValue }, ""),
                    latitude =
                    executeSafe({ (locationListener as MyLocationListener).latitudeValue }, ""),
                    isMockLocation = isMockLocationEnabled(
                        (locationListener as MyLocationListener).location
                    ),
                    locationPrecision = executeSafe({ (locationListener as MyLocationListener).precision }, 0f)
                )
            } else {
                return LocationInfoHolder()
            }
        }
        return LocationInfoHolder()
    }

    fun getLastKnownLocation(): LocationInfoHolder {
        if (ActivityCompat.checkSelfPermission(
                context,
                Manifest.permission.ACCESS_FINE_LOCATION
            ) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(
                    context,
                    Manifest.permission.ACCESS_COARSE_LOCATION
                ) == PackageManager.PERMISSION_GRANTED
        ) {
            val providers = lm.getProviders(true)
            var location: Location? = null
            providers.forEach {
                val tempLocation: Location? = lm.getLastKnownLocation(it)
                if (tempLocation != null) {
                    if (location == null || (tempLocation.accuracy.compareTo(location?.accuracy ?: 0f)) < 0) {
                        location = tempLocation
                    }
                }
            }
            location?.let {
                val longitude = "Longitude: " + it.longitude
                val latitude = "Latitude: " + it.latitude
                val precision = it.accuracy
                val locationInfo = LocationInfoHolder(
                    longitude =
                    executeSafe({ longitude }, ""),
                    latitude =
                    executeSafe({ latitude }, ""),
                    isMockLocation = isMockLocationEnabled(location),
                    locationPrecision = precision
                )

                lastKnownLocation = it
                return locationInfo
            }
            return LocationInfoHolder()
        } else {
            return LocationInfoHolder()
        }
    }

    private class MyLocationListener : LocationListener {
        var longitudeValue = ""
        var latitudeValue = ""
        var precision = 0f
        var location: Location? = null
        override fun onLocationChanged(loc: Location) {
            val longitude = "Longitude: " + loc.longitude
            val latitude = "Latitude: " + loc.latitude
            this.location = loc
            longitudeValue = longitude
            latitudeValue = latitude
            locationLatitude = latitudeValue
            locationLongitude = longitudeValue
            precision = loc.accuracy
        }

        override fun onProviderDisabled(provider: String) {}
        override fun onProviderEnabled(provider: String) {}
        override fun onStatusChanged(provider: String, status: Int, extras: Bundle) {}
    }

    companion object {
        var locationCityName = ""
        var locationLatitude = ""
        var locationLongitude = ""
        var precision = 0f
    }

    fun removeListener() {
        locationListener?.let { lm.removeUpdates(it) }
    }

    fun decodeCityName(location: Location?, context: Context): String {
        if (Geocoder.isPresent()) {
            location?.let {
                var cityName: String? = null
                val gcd = Geocoder(context, Locale.getDefault())
                val addresses: List<Address>
                try {
                    addresses = gcd.getFromLocation(
                        it.latitude,
                        it.longitude,
                        1
                    ) as List<Address>
                    if (addresses.isNotEmpty()) {
                        println(addresses[0].locality)
                        cityName = addresses[0].locality
                    }
                } catch (e: IOException) {
                    BureauAPI.handleException(e)
                    locationCityName = ""
                }
                locationCityName = cityName ?: ""
                return cityName ?: ""
            }
        }
        return ""
    }

    fun decodeCountryName(location: Location?, context: Context): String {
        if (Geocoder.isPresent()) {
            location?.let {
                var countryName: String? = null
                val gcd = Geocoder(context, Locale.getDefault())
                val addresses: List<Address>
                try {
                    addresses = gcd.getFromLocation(
                        it.latitude,
                        it.longitude,
                        1
                    ) as List<Address>
                    if (addresses.isNotEmpty()) {
                        countryName = addresses[0].countryName
                    }
                } catch (e: IOException) {
                    BureauAPI.handleException(e)
                    locationCityName = ""
                }
                locationCityName = countryName ?: ""
                return countryName ?: ""
            }
        }
        return ""
    }

    fun decodeRegionName(location: Location?, context: Context): String {
        if (Geocoder.isPresent()) {
            location?.let {
                var regionName: String? = null
                val gcd = Geocoder(context, Locale.getDefault())
                val addresses: List<Address>
                try {
                    addresses = gcd.getFromLocation(
                        it.latitude,
                        it.longitude,
                        1
                    ) as List<Address>
                    if (addresses.isNotEmpty()) {
                        regionName = addresses[0].adminArea
                    }
                } catch (e: IOException) {
                    BureauAPI.handleException(e)
                    locationCityName = ""
                }
                locationCityName = regionName ?: ""
                return regionName ?: ""
            }
        }
        return ""
    }
}
