package com.bureau.devicefingerprint.usecases

import android.content.Context
import android.os.Build
import com.bureau.devicefingerprint.security.SecureJNI
import com.bureau.devicefingerprint.tools.JSONUtils
import com.google.gson.GsonBuilder
import java.io.InputStream
import java.nio.charset.StandardCharsets
import java.security.KeyFactory
import java.security.MessageDigest
import java.security.PublicKey
import java.security.SecureRandom
import java.security.spec.X509EncodedKeySpec
import java.util.Base64
import java.util.Scanner
import javax.crypto.Cipher
import javax.crypto.spec.IvParameterSpec
import javax.crypto.spec.SecretKeySpec
import kotlin.text.Charsets.UTF_8

class DeviceDataEncryptionUseCase(private val context: Context) {
    private val md5HashValueKey = SecureJNI.getPKP1c()
    private val dummyKey = SecureJNI.getPKP2c()
    private val dummyValue = SecureJNI.getPKP3c()
    lateinit var ivGlobal: ByteArray
    lateinit var passPhrase: ByteArray

    fun initKeys() {
        ivGlobal = generateSecureRandomHexString(16).toByteArray(StandardCharsets.UTF_8)
        passPhrase = generateSecureRandomHexString(32).toByteArray(StandardCharsets.UTF_8)
    }

    private val TAG = "DeviceDataEncryption"

    private fun getPublicKey(): String {
        return readStream(
            context.assets.open("public_key.txt")
        )
    }

    private fun readStream(`is`: InputStream): String {
        val s = Scanner(`is`).useDelimiter("\\A")
        return if (s.hasNext()) s.next() else ""
    }

    private fun ByteArray.toHex() = joinToString(separator = "") { byte -> "%02x".format(byte) }

    private fun generateSecureRandomHexString(size: Int): String {
        val secureRandom = SecureRandom()
        val hexChars = "0123456789abcdef".toCharArray()
        val hexString = StringBuilder(size)
        for (i in 0 until size) {
            val randomIndex = secureRandom.nextInt(hexChars.size)
            hexString.append(hexChars[randomIndex])
        }
        return hexString.toString()
    }
    fun <T> getEncryptedPayload(payLoad: T): String {
        val sha256EncryptedPayLoad = getSHA256EncryptedPayLoad(payLoad) ?: ""
        return getAESEncryptedPayLoad(sha256EncryptedPayLoad)
    }

    private fun <T> getSHA256EncryptedPayLoad(payLoad: T): String? {
        val gson = GsonBuilder()
            .serializeNulls()
            .disableHtmlEscaping()
            .create()
        val payloadStringifiedBeforeSortingKeys: String = gson.toJson(payLoad)
        val payLoadTreeMap = JSONUtils.jsonToMap(payloadStringifiedBeforeSortingKeys)
        payLoadTreeMap.put(dummyKey, dummyValue)
        val payloadStringifiedAfterSortingKeys = gson.toJson(payLoadTreeMap)
        val crypt = MessageDigest.getInstance("SHA-256")
            .digest(payloadStringifiedAfterSortingKeys.toByteArray(UTF_8))

        payLoadTreeMap.put(md5HashValueKey, crypt.toHex())
        payLoadTreeMap.remove(dummyKey)
//        DebugUtils.d(TAG, "main: $payLoad")
//        DebugUtils.d(TAG, "main: $payloadStringifiedBeforeSortingKeys")
//        DebugUtils.d(TAG, "main: $payloadStringifiedAfterSortingKeys")

        return gson.toJson(payLoadTreeMap)
    }

    private fun encrypt(key: SecretKeySpec, data: ByteArray): ByteArray {
        val cipher = Cipher.getInstance("AES/CBC/PKCS5Padding")
        cipher.init(Cipher.ENCRYPT_MODE, key, IvParameterSpec(ivGlobal))
        return cipher.doFinal(data)
    }

    private fun arrayBufferToHexString(buffer: ByteArray): String {
        return buffer.joinToString("") { byte -> "%02x".format(byte) }
    }

    private val HEX_ARRAY = "0123456789ABCDEF".toByteArray(StandardCharsets.US_ASCII)
    private fun bytesToHex(bytes: ByteArray): String {
        val hexChars = ByteArray(bytes.size * 2)
        for (j in bytes.indices) {
            val v = bytes[j].toInt() and 0xFF
            hexChars[j * 2] = HEX_ARRAY[v ushr 4]
            hexChars[j * 2 + 1] = HEX_ARRAY[v and 0x0F]
        }
        return String(hexChars, StandardCharsets.UTF_8)
    }

    private fun getAESEncryptedPayLoad(payLoad: String): String {
        val key = SecretKeySpec(passPhrase, "AES")
        val data = payLoad.toByteArray(StandardCharsets.UTF_8)
        val encrypted = encrypt(key, data)
        // DebugUtils.d(TAG, "main1: $encryptedString")
        return bytesToHex(encrypted)
    }

    fun getRSAEncryptedString(jsonPayload: ByteArray): String {
        try {
            var publicKeyString =
                getPublicKey()
            publicKeyString = publicKeyString.replace("-----BEGIN PUBLIC KEY-----", "")
                .replace("-----END PUBLIC KEY-----", "").replace("\\s".toRegex(), "")
            val publicKeyBytes: ByteArray = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                Base64.getDecoder().decode(publicKeyString)
            } else {
                android.util.Base64.decode(
                    publicKeyString,
                    android.util.Base64.DEFAULT
                )
            }
            val publicKeySpec = X509EncodedKeySpec(publicKeyBytes)
            val keyFactory = KeyFactory.getInstance("RSA")
            val publicKey = keyFactory.generatePublic(publicKeySpec)

            return encrypt(publicKey, jsonPayload)
        } catch (e: Exception) {
            e.printStackTrace()
        }

        return ""
    }

    @Throws(java.lang.Exception::class)
    private fun encrypt(publicKey: PublicKey?, jsonPayload: ByteArray): String {
        val cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding")
        cipher.init(Cipher.ENCRYPT_MODE, publicKey)
        val encryptedBytes = cipher.doFinal(jsonPayload)
        val bundleAsBase64: String =
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                Base64.getEncoder().encodeToString(encryptedBytes)
            } else {
                android.util.Base64.encodeToString(
                    encryptedBytes,
                    android.util.Base64.DEFAULT
                )
            }
        return bundleAsBase64
    }
}
